@@ -24,14 +24,14 @@
 #include "region.h"
 
 static void
-maskbox(std::vector<char> &mask, int gridID, const SelboxInfo &selboxInfo)
+maskbox(Vmask &mask, int gridID, const SelboxInfo &selboxInfo)
 {
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   long nlon = gridInqXsize(gridID);
   long nlat = gridInqYsize(gridID);
 
@@ -44,7 +44,7 @@ maskbox(std::vector<char> &mask, int gridID, const SelboxInfo &selboxInfo)
 void getlonlatparams(int argc_offset, double &xlon1, double &xlon2, double &xlat1, double &xlat2);
 
 static void
-maskbox_cell(std::vector<char> &mask, int gridID)
+maskbox_cell(Vmask &mask, int gridID)
 {
   double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
   getlonlatparams(0, xlon1, xlon2, xlat1, xlat2);
@@ -68,20 +68,20 @@ maskbox_cell(std::vector<char> &mask, int gridID)
   if (xlat1 > xlat2) std::swap(xlat1, xlat2);
 
   for (size_t i = 0; i < gridsize; ++i)
-    {
-      mask[i] = true;
+  {
+    mask[i] = true;
 
-      auto xval = xvals[i];
-      auto yval = yvals[i];
-      if (yval >= xlat1 && yval <= xlat2)
-        {
-          if (((xval >= xlon1 && xval <= xlon2) || (xval - 360 >= xlon1 && xval - 360 <= xlon2)
-               || (xval + 360 >= xlon1 && xval + 360 <= xlon2)))
-            {
-              mask[i] = false;
-            }
-        }
+    auto xval = xvals[i];
+    auto yval = yvals[i];
+    if (yval >= xlat1 && yval <= xlat2)
+    {
+      if (((xval >= xlon1 && xval <= xlon2) || (xval - 360 >= xlon1 && xval - 360 <= xlon2)
+           || (xval + 360 >= xlon1 && xval + 360 <= xlon2)))
+      {
+        mask[i] = false;
+      }
     }
+  }
 
   if (gridID0 != gridID) gridDestroy(gridID);
 }
@@ -98,9 +98,9 @@ point_is_inside(double xval, double yval, size_t n, const double *xcoords, const
   auto c = false;
 
   for (size_t i = 0, j = n - 1; i < n; j = i++)
-    {
-      if (is_point_inside(xval, yval, xcoords[i], xcoords[j], ycoords[i], ycoords[j])) c = !c;
-    }
+  {
+    if (is_point_inside(xval, yval, xcoords[i], xcoords[j], ycoords[i], ycoords[j])) c = !c;
+  }
 
   return c;
 }
@@ -123,7 +123,7 @@ point_is_inside(double xval, double yval, double xmin, double xmax, const double
 }
 
 static void
-mask_region_regular(std::vector<char> &mask, size_t nlon, size_t nlat, const Varray<double> &xvals, const Varray<double> &yvals,
+mask_region_regular(Vmask &mask, size_t nlon, size_t nlat, Varray<double> const &xvals, Varray<double> const &yvals,
                     const double *xcoords, const double *ycoords, size_t segmentSize)
 {
   auto xmm = varray_min_max(segmentSize, xcoords);
@@ -134,19 +134,19 @@ mask_region_regular(std::vector<char> &mask, size_t nlon, size_t nlat, const Var
 #pragma omp parallel for schedule(static) default(shared)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
+  {
+    auto ilat = i / nlon;
+    auto yval = yvals[ilat];
+    if (yval > ymm.min && yval < ymm.max)
     {
-      auto ilat = i / nlon;
-      auto yval = yvals[ilat];
-      if (yval > ymm.min && yval < ymm.max)
-        {
-          if (point_is_inside(xvals[i - ilat * nlon], yval, xmm.min, xmm.max, xcoords, ycoords, segmentSize)) mask[i] = false;
-        }
+      if (point_is_inside(xvals[i - ilat * nlon], yval, xmm.min, xmm.max, xcoords, ycoords, segmentSize)) mask[i] = false;
     }
+  }
 }
 
 static void
-mask_region_cell(std::vector<char> &mask, size_t gridsize, const Varray<double> &xvals, const Varray<double> &yvals,
-                 const double *xcoords, const double *ycoords, size_t segmentSize)
+mask_region_cell(Vmask &mask, size_t gridsize, Varray<double> const &xvals, Varray<double> const &yvals, const double *xcoords,
+                 const double *ycoords, size_t segmentSize)
 {
   auto xmm = varray_min_max(segmentSize, xcoords);
   auto ymm = varray_min_max(segmentSize, ycoords);
@@ -155,13 +155,13 @@ mask_region_cell(std::vector<char> &mask, size_t gridsize, const Varray<double>
 #pragma omp parallel for schedule(static) default(shared)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
+  {
+    auto yval = yvals[i];
+    if (yval >= ymm.min && yval <= ymm.max)
     {
-      auto yval = yvals[i];
-      if (yval >= ymm.min && yval <= ymm.max)
-        {
-          if (point_is_inside(xvals[i], yval, xmm.min, xmm.max, xcoords, ycoords, segmentSize)) mask[i] = false;
-        }
+      if (point_is_inside(xvals[i], yval, xmm.min, xmm.max, xcoords, ycoords, segmentSize)) mask[i] = false;
     }
+  }
 }
 
 static int
@@ -169,28 +169,28 @@ get_gridID(int vlistID1, bool operIndexBox)
 {
   std::vector<int> gridsFound;
 
-  auto ngrids = vlistNumGrids(vlistID1);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID1);
+  for (int index = 0; index < numGrids; ++index)
+  {
+    auto gridID1 = vlistGrid(vlistID1, index);
+    if (gridInqSize(gridID1) == 1) continue;
+
+    auto gridtype = gridInqType(gridID1);
+    auto projtype = gridInqProjType(gridID1);
+
+    auto isReg2dGeoGrid = (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR);
+    auto projHasGeoCoords = (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL);
+
+    if (isReg2dGeoGrid || projHasGeoCoords || (operIndexBox && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION))
+        || (!operIndexBox && (gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID1))))
     {
-      auto gridID1 = vlistGrid(vlistID1, index);
-      if (gridInqSize(gridID1) == 1) continue;
-
-      auto gridtype = gridInqType(gridID1);
-      auto projtype = gridInqProjType(gridID1);
-
-      auto isReg2dGeoGrid = (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR);
-      auto projHasGeoCoords = (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL);
-
-      if (isReg2dGeoGrid || projHasGeoCoords || (operIndexBox && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION))
-          || (!operIndexBox && (gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID1))))
-        {
-          gridsFound.push_back(gridID1);
-        }
-      else
-        {
-          if (gridInqSize(gridID1) > 2) cdo_warning("Unsupported grid type: %s", gridNamePtr(gridtype));
-        }
+      gridsFound.push_back(gridID1);
     }
+    else
+    {
+      if (gridInqSize(gridID1) > 2) cdo_warning("Unsupported grid type: %s", gridNamePtr(gridtype));
+    }
+  }
 
   if (gridsFound.size() == 0) cdo_abort("No processable grid found!");
   if (gridsFound.size() > 1) cdo_abort("Too many different grids!");
@@ -200,7 +200,7 @@ get_gridID(int vlistID1, bool operIndexBox)
 }
 
 static std::vector<bool>
-get_processVars(const VarList &varList, int gridID)
+get_processVars(VarList const &varList, int gridID)
 {
   auto numVars = varList.numVars();
 
@@ -235,18 +235,18 @@ public:
   };
   inline static auto registration = RegisterEntry<Maskbox>(module);
 
-  int MASKLONLATBOX, MASKINDEXBOX, MASKREGION, MASKCIRCLE;
+  int MASKLONLATBOX{}, MASKINDEXBOX{}, MASKREGION{}, MASKCIRCLE{};
 
 private:
-  CdoStreamID streamID1;
-  CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
-  std::vector<bool> processVars;
-  std::vector<char> mask;
-  size_t gridsize;
+  CdoStreamID streamID1{};
+  CdoStreamID streamID2{};
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  std::vector<bool> processVars{};
+  Vmask mask{};
+  size_t gridsize{};
 
-  VarList varList1;
+  VarList varList1{};
 
 public:
   void
@@ -280,84 +280,83 @@ public:
     operator_input_arg(cdo_operator_enter(operatorID));
 
     gridsize = gridInqSize(gridID);
-    mask = std::vector<char>(gridsize, true);
+    mask.resize(gridsize, true);
 
     auto gridtype = gridInqType(gridID);
 
     if (operatorID == MASKLONLATBOX)
-      {
-        if (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID))
-          maskbox_cell(mask, gridID);
-        else
-          maskbox(mask, gridID, gen_lonlat_selbox(0, gridID));
-      }
+    {
+      if (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID))
+        maskbox_cell(mask, gridID);
+      else
+        maskbox(mask, gridID, gen_lonlat_selbox(0, gridID));
+    }
     else if (operatorID == MASKINDEXBOX) { maskbox(mask, gridID, gen_index_selbox(0, gridID)); }
     else if (operatorID == MASKREGION)
+    {
+      auto nlon = gridInqXsize(gridID);
+      auto nlat = gridInqYsize(gridID);
+      auto fullGrid = (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID));
+      Varray<double> xvals(fullGrid ? gridsize : nlon), yvals(fullGrid ? gridsize : nlat);
+
+      auto gridID0 = gridID;
+      if (fullGrid)
       {
-        auto nlon = gridInqXsize(gridID);
-        auto nlat = gridInqYsize(gridID);
-        auto fullGrid = (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID));
-        Varray<double> xvals(fullGrid ? gridsize : nlon), yvals(fullGrid ? gridsize : nlat);
-
-        auto gridID0 = gridID;
-        if (fullGrid)
-          {
-            gridID = generate_full_grid(gridID);
-            if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
-          }
-
-        gridInqXvals(gridID, xvals.data());
-        gridInqYvals(gridID, yvals.data());
-
-        // Convert lat/lon units if required
-        cdo_grid_to_degree(gridID, CDI_XAXIS, xvals, "grid center lon");
-        cdo_grid_to_degree(gridID, CDI_YAXIS, yvals, "grid center lat");
-
-        auto numFiles = cdo_operator_argc();
-        for (int i = 0; i < numFiles; ++i)
-          {
-            Regions regions;
-            auto param = cdo_operator_argv(i);
-            if (param.starts_with("dcw:"))
-              read_regions_from_dcw(param.c_str() + 4, regions);
-            else
-              read_regions_from_file(param, regions);
-
-            for (size_t k = 0; k < regions.numSegments; ++k)
-              {
-                auto segmentSize = regions.segmentSize[k];
-                if (segmentSize < 3) continue;
-                auto offset = regions.segmentOffset[k];
-                auto xcoords = &regions.x[offset];
-                auto ycoords = &regions.y[offset];
-                if (fullGrid)
-                  mask_region_cell(mask, gridsize, xvals, yvals, xcoords, ycoords, segmentSize);
-                else
-                  mask_region_regular(mask, nlon, nlat, xvals, yvals, xcoords, ycoords, segmentSize);
-              }
-          }
-
-        if (gridID0 != gridID) gridDestroy(gridID);
+        gridID = generate_full_grid(gridID);
+        if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
       }
+
+      gridInqXvals(gridID, xvals.data());
+      gridInqYvals(gridID, yvals.data());
+
+      // Convert lat/lon units if required
+      cdo_grid_to_degree(gridID, CDI_XAXIS, xvals, "grid center lon");
+      cdo_grid_to_degree(gridID, CDI_YAXIS, yvals, "grid center lat");
+
+      auto numFiles = cdo_operator_argc();
+      for (int i = 0; i < numFiles; ++i)
+      {
+        Regions regions;
+        auto param = cdo_operator_argv(i);
+        if (param.starts_with("dcw:"))
+          read_regions_from_dcw(param.c_str() + 4, regions);
+        else
+          read_regions_from_file(param, regions);
+
+        for (size_t k = 0; k < regions.numSegments; ++k)
+        {
+          auto segmentSize = regions.segmentSize[k];
+          if (segmentSize < 3) continue;
+          auto offset = regions.segmentOffset[k];
+          auto xcoords = &regions.x[offset];
+          auto ycoords = &regions.y[offset];
+          if (fullGrid)
+            mask_region_cell(mask, gridsize, xvals, yvals, xcoords, ycoords, segmentSize);
+          else
+            mask_region_regular(mask, nlon, nlat, xvals, yvals, xcoords, ycoords, segmentSize);
+        }
+      }
+
+      if (gridID0 != gridID) gridDestroy(gridID);
+    }
     else if (operatorID == MASKCIRCLE)
-      {
-        /*
-        CirclePoint cpoint;
+    {
+      /*
+      CirclePoint cpoint;
 
-        selcircle_get_parameter(cpoint);
-        if (cpoint.radius < 0.0 || cpoint.radius > 180.0) cdo_abort("radius=%g out of bounds (0-180 deg)!", cpoint.radius);
+      selcircle_get_parameter(cpoint);
+      if (cpoint.radius < 0.0 || cpoint.radius > 180.0) cdo_abort("radius=%g out of bounds (0-180 deg)!", cpoint.radius);
 
-        auto gridsizemax = vlistGridsizeMax(vlistID1);
-        if (gridsizemax < cpoint.maxpoints) cpoint.maxpoints = gridsizemax;
-        if (Options::cdoVerbose)
-          cdo_print("lon = %g, lat = %g, radius = %gdeg(%gkm)", cpoint.lon, cpoint.lat, cpoint.radius,
-                    radiusDegToKm(cpoint.radius));
+      if (varList1.gridsizeMax() < cpoint.maxpoints) cpoint.maxpoints = varList1.gridsizeMax();
+      if (Options::cdoVerbose)
+        cdo_print("lon = %g, lat = %g, radius = %gdeg(%gkm)", cpoint.lon, cpoint.lat, cpoint.radius,
+                  radiusDegToKm(cpoint.radius));
 
-        cpoint.radius *= DEG2RAD;
-        cpoint.lon *= DEG2RAD;
-        cpoint.lat *= DEG2RAD;
-        */
-      }
+      cpoint.radius *= DEG2RAD;
+      cpoint.lon *= DEG2RAD;
+      cpoint.lat *= DEG2RAD;
+      */
+    }
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
@@ -370,43 +369,38 @@ public:
 
     int tsID = 0;
     while (true)
+    {
+      auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+      if (numFields == 0) break;
+
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
+      cdo_def_timestep(streamID2, tsID);
+
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto [varID, levelID] = cdo_inq_field(streamID1);
 
-        cdo_taxis_copy_timestep(taxisID2, taxisID1);
-        cdo_def_timestep(streamID2, tsID);
+        if (processVars[varID])
+        {
+          auto const &var = varList1.vars[varID];
+          field.init(var);
+          cdo_read_field(streamID1, field);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+          auto func = [&](auto &v)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            for (size_t i = 0; i < gridsize; ++i)
+              if (mask[i]) v[i] = var.missval;
+          };
+          field_operation(func, field);
 
-            if (processVars[varID])
-              {
-                const auto &var = varList1.vars[varID];
-                field.init(var);
-                cdo_read_record(streamID1, field);
-
-                if (field.memType == MemType::Float)
-                  {
-                    for (size_t i = 0; i < gridsize; ++i)
-                      if (mask[i]) field.vec_f[i] = var.missval;
-                  }
-                else
-                  {
-                    for (size_t i = 0; i < gridsize; ++i)
-                      if (mask[i]) field.vec_d[i] = var.missval;
-                  }
-
-                field_num_mv(field);
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, field);
-              }
-          }
-
-        tsID++;
+          field_num_mv(field);
+          cdo_def_field(streamID2, varID, levelID);
+          cdo_write_field(streamID2, field);
+        }
       }
+
+      tsID++;
+    }
   }
 
   void
