@@ -7,7 +7,7 @@
 #endif
 
 #ifdef HAVE_LIBNETCDF
-#include "netcdf.h"
+#include <netcdf.h>
 #endif
 
 #include "dcw_reader.h"
@@ -45,7 +45,7 @@ dcw_get_path(const char *name, const char *suffix, char *path)
 
   if (DCW_Dir)
     {
-      sprintf(path, "%s/%s%s", DCW_Dir, name, suffix);
+      std::snprintf(path, PATH_MAX, "%s/%s%s", DCW_Dir, name, suffix);
       if (access(path, R_OK) == 0) found = true;
     }
   else
@@ -73,13 +73,13 @@ dcw_load_lists(DCW_Lists &dcw_lists)
   size_t n_alloc = 300;
   unsigned int k, n;
   char line[BUFSIZ] = { 0 };
-  struct DCW_Country_State *Country_State = NULL;
+  // DCW_Country_State *Country_State = NULL;
 
   char path[PATH_MAX] = { 0 };
   if (!dcw_get_path("dcw-countries", ".txt", path)) return -1;
 
   // Get countries first
-  auto fp = fopen(path, "r");
+  auto fp = std::fopen(path, "r");
   if (fp == nullptr)
     {
       fprintf(stderr, "Unable to open file %s [permission trouble?]\n", path);
@@ -92,7 +92,7 @@ dcw_load_lists(DCW_Lists &dcw_lists)
   while (fgets(line, BUFSIZ, fp))
     {
       if (line[0] == '#') continue;  // Skip comments
-      sscanf(line, "%s %s %[^\n]", countries[k].continent, countries[k].code, countries[k].name);
+      std::sscanf(line, "%s %s %[^\n]", countries[k].continent, countries[k].code, countries[k].name);
       k++;
       if (k == n_alloc)
         {
@@ -101,19 +101,16 @@ dcw_load_lists(DCW_Lists &dcw_lists)
         }
     }
 
-  fclose(fp);
+  std::fclose(fp);
 
   dim[0] = k;  // Number of countries
 
   countries.resize(k);
 
   // Get states
-  if (!dcw_get_path("dcw-states", ".txt", path))
-    {
-      return -1;
-    }
+  if (!dcw_get_path("dcw-states", ".txt", path)) { return -1; }
 
-  fp = fopen(path, "r");
+  fp = std::fopen(path, "r");
   if (fp == nullptr)
     {
       fprintf(stderr, "Unable to open file %s [permission trouble?]\n", path);
@@ -127,8 +124,8 @@ dcw_load_lists(DCW_Lists &dcw_lists)
   while (fgets(line, BUFSIZ, fp))
     {
       if (line[0] == '#') continue;  // Skip comments
-      sscanf(line, "%s %s %[^\n]", states[k].country, states[k].code, states[k].name);
-      if (k && strcmp(states[k].country, states[k - 1].country)) n++;  // New country with states
+      std::sscanf(line, "%s %s %[^\n]", states[k].country, states[k].code, states[k].name);
+      if (k && std::strcmp(states[k].country, states[k - 1].country)) n++;  // New country with states
       k++;
       if (k == n_alloc)
         {
@@ -137,24 +134,26 @@ dcw_load_lists(DCW_Lists &dcw_lists)
         }
     }
 
-  fclose(fp);
+  std::fclose(fp);
 
   dim[1] = k;  // Number of states
   states.resize(k);
 
   // Get list of countries with states
 
-  dim[2] = n;    // Number of countries with states
-  if (0 /*CS*/)  // Wants list returned
-    {
-      Country_State = (DCW_Country_State *) malloc(n * sizeof(struct DCW_Country_State));
-      memcpy(Country_State[0].country, states[0].country, 4);
-      for (k = n = 1; k < dim[1]; k++)
-        {
-          if (strcmp(states[k].country, states[k - 1].country)) memcpy(Country_State[n++].country, states[k].country, 4);
-        }
-      // *CS = Country_State;
-    }
+  dim[2] = n;  // Number of countries with states
+  /*
+  if (0)  // Wants list returned
+  {
+    Country_State = (DCW_Country_State *) malloc(n * sizeof(struct DCW_Country_State));
+    std::memcpy(Country_State[0].country, states[0].country, 4);
+    for (k = n = 1; k < dim[1]; ++k)
+      {
+        if (std::strcmp(states[k].country, states[k - 1].country)) std::memcpy(Country_State[n++].country, states[k].country, 4);
+      }
+    // *CS = Country_State;
+  }
+  */
 
   // fprintf(stderr, "# DCW: Found %u countries, %u countries with states, and %u states\n", dim[0], dim[2], dim[1]);
 
@@ -162,16 +161,16 @@ dcw_load_lists(DCW_Lists &dcw_lists)
 }
 
 static int
-dcw_find_country(const std::string &code, const std::vector<DCW_Country> &list)
+dcw_find_country(std::string const &code, const std::vector<DCW_Country> &list)
 {
   int low = 0, high = (int) list.size() - 1;
 
   while (low <= high)
     {
-      const auto midpoint = low + (high - low) / 2;
+      auto midpoint = low + (high - low) / 2;
 
       // check to see if value is equal to item in array
-      const auto way = strcmp(code.c_str(), list[midpoint].code);
+      auto way = std::strcmp(code.c_str(), list[midpoint].code);
       if (way == 0) return midpoint;
 
       if (way < 0)
@@ -184,12 +183,26 @@ dcw_find_country(const std::string &code, const std::vector<DCW_Country> &list)
   return -1;
 }
 
+static int
+dcw_find_state(std::string const &code, const std::vector<DCW_State> &list)
+{
+  auto country = code.substr(0, 2);
+  auto state = code.substr(2);
+  for (size_t i = 0, n = list.size(); i < n; ++i)
+    {
+      if (country == list[i].country && state == list[i].code) return i;
+    }
+
+  // item was not found
+  return -1;
+}
+
 #ifdef HAVE_LIBNETCDF
 static int
-nc_get_minmax(int ncid, const char *name, double &minval, double &maxval)
+nc_get_minmax(int ncid, std::string const &name, double &minval, double &maxval)
 {
   int varid;
-  if (nc_inq_varid(ncid, name, &varid)) return 1;
+  if (nc_inq_varid(ncid, name.c_str(), &varid)) return 1;
   if (nc_get_att_double(ncid, varid, "min", &minval)) return 1;
   if (nc_get_att_double(ncid, varid, "max", &maxval)) return 1;
 
@@ -219,45 +232,70 @@ dcw_open_nc()
 }
 
 int
-dcw_get_region(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList, Region &region)
+dcw_get_region(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList, Region &region)
 {
-  const auto ncid = dcw_open_nc();
+  auto ncid = dcw_open_nc();
   if (ncid < 0) return 1;
 
+  int numEntries = 0;
   for (auto &code : codeList)
     {
-      auto ks = dcw_find_country(code, dcw_lists.countries);
+      auto ks = (code.size() > 2) ? dcw_find_state(code, dcw_lists.states) : dcw_find_country(code, dcw_lists.countries);
       if (ks == -1)
         {
-          printf("No country code matching %s (skipped)\n", code.c_str());
-          continue;
+          fprintf(stderr, "No country code matching <%s>!\n", code.c_str());
+          return 3;
         }
 
       // auto &country = dcw_lists.countries[ks];
-      // const auto is_Antarctica = (!strncmp(country.code, "AQ", 2U));
+      // auto is_Antarctica = (!std::strncmp(country.code, "AQ", 2U));
 
       auto xname = code + "_lon";
       auto yname = code + "_lat";
 
       double west = 0.0, east = 0.0, south = 0.0, north = 0.0;
 #ifdef HAVE_LIBNETCDF
-      if (nc_get_minmax(ncid, xname.c_str(), west, east)) continue;
-      if (nc_get_minmax(ncid, yname.c_str(), south, north)) continue;
+      if (nc_get_minmax(ncid, xname, west, east)) continue;
+      if (nc_get_minmax(ncid, yname, south, north)) continue;
 #endif
       // if (west <= 0 && east >=0) fprintf(stderr, "%s: %g %g\n", code.c_str(), west, east);
       // if (west >= 180 && east >=180) fprintf(stderr, "%s: %g %g\n", code.c_str(), west, east);
-      if (west >= 180.0) west -= 360.0;
-      if (east > 180.0) east -= 360.0;
+      if (west >= 180.0 && east > 180.0)
+        {
+          west -= 360.0;
+          east -= 360.0;
+        }
+      if (numEntries > 0)
+        {
+          if (east <= 0.0 && region.west >= 0.0)
+            {
+              west += 360.0;
+              east += 360.0;
+            }
+          else if (east >= 180.0 && region.east <= 0.0)
+            {
+              region.west += 360.0;
+              region.east += 360.0;
+            }
+        }
       region.west = std::min(region.west, west);
       region.south = std::min(region.south, south);
       region.east = std::max(region.east, east);
       region.north = std::max(region.north, north);
+
+      numEntries++;
     }
 
 #ifdef HAVE_LIBNETCDF
   nc_close(ncid);
 #endif
 
+  if (numEntries == 0)
+    {
+      printf("Empty code list!\n");
+      return 2;
+    }
+
   return 0;
 }
 /*
@@ -285,7 +323,7 @@ nc_get_data(int ncid, const char *name, std::vector<double> &data)
 
   data.resize(np);
   scale = 1.0 / scale;
-  for (size_t i = 0; i < np; i++)  // Unpack
+  for (size_t i = 0; i < np; ++i)  // Unpack
     data[i] = (shortData[i] == 65535U) ? 0.0 : shortData[i] * scale + minval;
 
   return 0;
@@ -294,17 +332,17 @@ nc_get_data(int ncid, const char *name, std::vector<double> &data)
 */
 #ifdef HAVE_LIBNETCDF
 static int
-nc_get_lonlat(int ncid, const char *xname, const char *yname, std::vector<double> &x, std::vector<double> &y)
+nc_get_lonlat(int ncid, std::string const &xname, std::string const &yname, std::vector<double> &x, std::vector<double> &y)
 {
   int xvarid;
   double xscale, xmin;
-  if (nc_inq_varid(ncid, xname, &xvarid)) return 1;
+  if (nc_inq_varid(ncid, xname.c_str(), &xvarid)) return 1;
   if (nc_get_att_double(ncid, xvarid, "min", &xmin)) return 1;
   if (nc_get_att_double(ncid, xvarid, "scale", &xscale)) return 1;
 
   int yvarid;
   double yscale, ymin;
-  if (nc_inq_varid(ncid, yname, &yvarid)) return 1;
+  if (nc_inq_varid(ncid, yname.c_str(), &yvarid)) return 1;
   if (nc_get_att_double(ncid, yvarid, "min", &ymin)) return 1;
   if (nc_get_att_double(ncid, yvarid, "scale", &yscale)) return 1;
 
@@ -325,7 +363,7 @@ nc_get_lonlat(int ncid, const char *xname, const char *yname, std::vector<double
   y.resize(np);
   xscale = 1.0 / xscale;
   yscale = 1.0 / yscale;
-  for (size_t i = 0; i < np; i++)  // Unpack
+  for (size_t i = 0; i < np; ++i)  // Unpack
     {
       x[i] = (xshort[i] == 65535U) ? 0.0 : xshort[i] * xscale + xmin;
       y[i] = (xshort[i] == 65535U) ? 0.0 : yshort[i] * yscale + ymin;
@@ -337,74 +375,84 @@ nc_get_lonlat(int ncid, const char *xname, const char *yname, std::vector<double
 #endif
 
 int
-dcw_get_lonlat(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList, std::vector<double> &lon,
+dcw_get_lonlat(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList, std::vector<double> &lon,
                std::vector<double> &lat)
 {
-  const auto ncid = dcw_open_nc();
+  auto ncid = dcw_open_nc();
   if (ncid < 0) return 1;
 
-  size_t n = 0;
+  size_t numVals = 0;
+  int numEntries = 0;
   for (auto &code : codeList)
     {
-      auto ks = dcw_find_country(code, dcw_lists.countries);
+      auto ks = (code.size() > 2) ? dcw_find_state(code, dcw_lists.states) : dcw_find_country(code, dcw_lists.countries);
       if (ks == -1)
         {
-          printf("No country code matching %s (skipped)\n", code.c_str());
-          continue;
+          fprintf(stderr, "No country code matching <%s>!\n", code.c_str());
+          return 3;
         }
 
       // auto &country = dcw_lists.countries[ks];
-      // const auto is_Antarctica = (!strncmp(country.code, "AQ", 2U));
+      // auto is_Antarctica = (!std::strncmp(country.code, "AQ", 2U));
 
       auto xname = code + "_lon";
       auto yname = code + "_lat";
 
       std::vector<double> x, y;
 #ifdef HAVE_LIBNETCDF
-      if (nc_get_lonlat(ncid, xname.c_str(), yname.c_str(), x, y)) continue;
+      if (nc_get_lonlat(ncid, xname, yname, x, y)) continue;
 #endif
-      const auto offset = lon.size();
-      n += x.size();
-      lon.resize(n);
-      lat.resize(n);
-      for (size_t i = 0; i < x.size(); ++i) lon[offset + i] = x[i];
-      for (size_t i = 0; i < y.size(); ++i) lat[offset + i] = y[i];
+      auto offset = lon.size();
+      numVals += x.size();
+      lon.resize(numVals);
+      lat.resize(numVals);
+      for (size_t i = 0, n = x.size(); i < n; ++i) lon[offset + i] = x[i];
+      for (size_t i = 0, n = y.size(); i < n; ++i) lat[offset + i] = y[i];
+
+      numEntries++;
     }
 
 #ifdef HAVE_LIBNETCDF
   nc_close(ncid);
 #endif
 
+  if (numEntries == 0)
+    {
+      printf("Empty code list!\n");
+      return 2;
+    }
+
   return 0;
 }
 
 static int
-dcw_print_lonlat(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList)
+dcw_print_lonlat(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList)
 {
-  const auto ncid = dcw_open_nc();
+  auto ncid = dcw_open_nc();
   if (ncid < 0) return 1;
 
+  int numEntries = 0;
   for (auto &code : codeList)
     {
-      auto ks = dcw_find_country(code, dcw_lists.countries);
+      auto ks = (code.size() > 2) ? dcw_find_state(code, dcw_lists.states) : dcw_find_country(code, dcw_lists.countries);
       if (ks == -1)
         {
-          printf("No country code matching %s (skipped)\n", code.c_str());
-          continue;
+          printf("No country code matching <%s>!\n", code.c_str());
+          return 3;
         }
 
       auto &country = dcw_lists.countries[ks];
-      // const auto is_Antarctica = (!strncmp(country.code, "AQ", 2U));
+      // auto is_Antarctica = (!std::strncmp(country.code, "AQ", 2U));
 
       auto xname = code + "_lon";
       auto yname = code + "_lat";
 
       std::vector<double> x, y;
 #ifdef HAVE_LIBNETCDF
-      if (nc_get_lonlat(ncid, xname.c_str(), yname.c_str(), x, y)) continue;
+      if (nc_get_lonlat(ncid, xname, yname, x, y)) continue;
 #endif
       size_t nseg = 0;
-      const auto n = x.size();
+      auto n = x.size();
       for (size_t i = 0; i < n; ++i)
         {
           if (x[i] == 0.0 && y[i] == 0.0)
@@ -412,49 +460,51 @@ dcw_print_lonlat(const DCW_Lists &dcw_lists, const std::vector<std::string> &cod
           else
             printf("%.12g  %.12g\n", x[i], y[i]);
         }
+
+      numEntries++;
     }
 
 #ifdef HAVE_LIBNETCDF
   nc_close(ncid);
 #endif
 
+  if (numEntries == 0)
+    {
+      printf("Empty code list!\n");
+      return 2;
+    }
+
   return 0;
 }
 
 static bool
 compare_code(const DCW_Country &a, const DCW_Country &b)
 {
-  return (strcmp(a.code, b.code) < 0);
+  return (std::strcmp(a.code, b.code) < 0);
 }
 
 std::vector<std::string>
-dcw_expand_code_list(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList)
+dcw_expand_code_list(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList)
 {
   std::vector<std::string> codeListExpand;
   codeListExpand.reserve(codeList.size());
 
   for (auto &code : codeList)
     {
-      if (code.size() == 2)
-        {
-          codeListExpand.push_back(code);
-        }
+      if (code.size() == 2 || code.size() == 4 || code.size() == 5) { codeListExpand.push_back(code); }
       else if (code.size() == 3 && code[0] == '=')
         {
           int n = 0;
           for (auto &country : dcw_lists.countries)
             {
-              if (strncmp(country.continent, &code[1], 2)) continue;  // Not this one
+              if (std::strncmp(country.continent, &code[1], 2)) continue;  // Not this one
               codeListExpand.push_back(country.code);
               n++;
             }
 
           if (n == 0) fprintf(stderr, "Wrong DCW country or continent code <%s>\n", code.c_str() + 1);
         }
-      else
-        {
-          fprintf(stderr, "Wrong DCW country or continent code <%s>\n", code.c_str());
-        }
+      else { fprintf(stderr, "Wrong DCW country or continent code <%s>\n", code.c_str()); }
     }
 
   return codeListExpand;
@@ -487,7 +537,7 @@ dcw_print_states(const DCW_Lists &dcw_lists)
 }
 
 void
-dcw_print_polygons(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList)
+dcw_print_polygons(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList)
 {
   if (dcw_print_lonlat(dcw_lists, codeList)) fprintf(stderr, "dcw_print_lonlat failed!\n");
 }
@@ -495,15 +545,15 @@ dcw_print_polygons(const DCW_Lists &dcw_lists, const std::vector<std::string> &c
 void
 dcw_sort_countries(DCW_Lists &dcw_lists)
 {
-  std::sort(dcw_lists.countries.begin(), dcw_lists.countries.end(), compare_code);
+  std::ranges::sort(dcw_lists.countries, compare_code);
 }
 
 /*
 static void
-analyze_segments(const std::vector<double> &lon, const std::vector<double> &lat)
+analyze_segments(std::vector<double> const &lon, std::vector<double> const &lat)
 {
   size_t nseg = 0;
-  const auto n = lon.size();
+  auto n = lon.size();
   size_t sbegin = n;
   size_t send = 0;
   for (size_t i = 0; i < n; ++i)
@@ -534,10 +584,10 @@ analyze_segments(const std::vector<double> &lon, const std::vector<double> &lat)
 }
 
 static void
-dump_lonlat(const std::vector<double> &lon, const std::vector<double> &lat)
+dump_lonlat(std::vector<double> const &lon, std::vector<double> const &lat)
 {
   size_t nseg = 0;
-  const auto n = lon.size();
+  auto n = lon.size();
   for (size_t i = 0; i < n; ++i)
     {
       if (lon[i] == 0.0 && lat[i] == 0.0)
@@ -548,7 +598,7 @@ dump_lonlat(const std::vector<double> &lon, const std::vector<double> &lat)
 }
 
 static void
-print_lonlat(const DCW_Lists &dcw_lists, const std::vector<std::string> &codeList)
+print_lonlat(const DCW_Lists &dcw_lists, std::vector<std::string> const &codeList)
 {
   std::vector<double> lon, lat;
   if (dcw_get_lonlat(dcw_lists, codeList, lon, lat)) fprintf(stderr, "dcw_get_lonlat failed!\n");
@@ -592,7 +642,7 @@ int main(void)
   codeList = dcw_expand_code_list(dcw_lists, codeList);
 
   Region region;
-  if (dcw_get_region(dcw_lists, codeList, region)) fprintf(stderr, "dcw_get_region failed!\n");
+  if (dcw_get_region(dcw_lists, codeList, region)) fprintf(stderr, "dcw_get_region() failed!\n");
 
   printf("#   West=%g  East=%g  South=%g  North=%g\n", region.west, region.east, region.south, region.north);
   printf("#\n");
