@@ -2,9 +2,9 @@
 #define GRID_READ_PINGO_H
 
 #include <cstdio>
-#include <vector>
+#include "varray.h"
 
-size_t input_darray(FILE *gfp, size_t n_values, std::vector<double> &array);
+size_t input_darray(FILE *gfp, size_t n_values, Varray<double> &array);
 int grid_read_pingo(FILE *gfp);
 
 #endif
